var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "@gluestack-v2/framework-cli/build/plugin/BaseGluestackPluginInstance", "@gluestack-v2/framework-cli/build/constants/gluestack.v2", "path", "./helpers/write-sdk"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.PluginInstance = void 0;
    const BaseGluestackPluginInstance_1 = __importDefault(require("@gluestack-v2/framework-cli/build/plugin/BaseGluestackPluginInstance"));
    const gluestack_v2_1 = require("@gluestack-v2/framework-cli/build/constants/gluestack.v2");
    const path_1 = require("path");
    const write_sdk_1 = __importDefault(require("./helpers/write-sdk"));
    class PluginInstance extends BaseGluestackPluginInstance_1.default {
        constructor(app, callerPlugin, name, gluePluginStore, installationPath) {
            super(app, callerPlugin, name, gluePluginStore, installationPath);
        }
        init() {
            //
        }
        destroy() {
            //
        }
        getName() {
            return this.name;
        }
        getCallerPlugin() {
            return this.callerPlugin;
        }
        getIgnoredPaths() {
            return ['middlewares', 'events', 'private'];
        }
        watch(callback) {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.buildBeforeWatch();
                this.app.watch(this._sourcePath, this._destinationPath, (event, path) => __awaiter(this, void 0, void 0, function* () {
                    // TODO: OPTIMIZE UPDATES
                    console.log(event, path);
                    this.generateFunctionsInServiceGateway();
                    const packagePath = yield this.createClientSDKPackage();
                    this.buildPackage(packagePath);
                    // this.generateFunctionsInServiceSdk(this.getIgnoredPaths());
                    if (callback) {
                        callback(event, path);
                    }
                }));
            });
        }
        getSourcePath() {
            return (0, path_1.join)(process.cwd(), this.getPluginEnvironment(), this.getName());
        }
        getPluginEnvironment() {
            // @ts-ignore
            return this.callerPlugin.getPluginEnvironment();
        }
        generateFunctionsInServiceGateway() {
            const name = this.getName();
            const installationPath = this._destinationPath;
            const plugin = this.app.getPluginByName('@gluestack-v2/glue-plugin-service-gateway');
            // @ts-ignore
            plugin.generateService(installationPath, name);
        }
        generateFunctionsInServiceSdk(ignoredPaths) {
            const plugin = this.app.getPluginByName('@gluestack-v2/glue-plugin-service-sdk');
            // @ts-ignore
            plugin.generateSDK(this._sourcePath, this.getName(), ignoredPaths);
        }
        createClientSDKPackage() {
            return __awaiter(this, void 0, void 0, function* () {
                const clientSdkPackageName = `${this.getName()}-client-sdk`;
                const packagePath = (0, path_1.join)(gluestack_v2_1.GLUE_GENERATED_PACKAGES_PATH, clientSdkPackageName);
                const sdkPath = (0, path_1.join)(this.callerPlugin.getPackagePath(), 'sdk', 'client');
                yield this.app.createPackage(clientSdkPackageName, sdkPath);
                yield (0, write_sdk_1.default)(packagePath, this._sourcePath, sdkPath, [], this.getName());
                yield this.app.updateNameInPackageJSON(packagePath, clientSdkPackageName);
                return packagePath;
            });
        }
        prepare() {
            return __awaiter(this, void 0, void 0, function* () {
                //@ts-ignore
                const clientSdkPackageName = `${this.getName()}-client-sdk`;
                const packagePath = (0, path_1.join)(gluestack_v2_1.GLUE_GENERATED_PACKAGES_PATH, clientSdkPackageName);
                this.buildPackage(packagePath);
                const plugin = this.app.getPluginByName('@gluestack-v2/glue-plugin-sdk');
                yield plugin.build();
            });
        }
        build() {
            return __awaiter(this, void 0, void 0, function* () {
                yield this.app.write(this._sourcePath, this._destinationPath);
                // @ts-ignore
                this.generateFunctionsInServiceGateway();
                yield this.createClientSDKPackage();
                // this.createServerSDKPackage();
                // FIX: Do we need to prepare while building?
                yield this.prepare();
                // this.generateFunctionsInServiceSdk(this.getIgnoredPaths());
            });
        }
        getGatewayInstanceInfo() {
            const plugin = this.app.getPluginByName('@gluestack-v2/glue-plugin-service-gateway');
            if (!plugin) {
                console.error(`Plugin "@gluestack-v2/glue-plugin-service-gateway" not found.`);
                return '';
            }
            const instances = plugin.instances;
            if (!instances || instances.length <= 0) {
                console.error(`No instance with "@gluestack-v2/glue-plugin-service-gateway" found.`);
                return '';
            }
            return instances[0].getName();
        }
        // Override getDestinationPath
        getDestinationPath() {
            const gatewayInstanceName = this.getGatewayInstanceInfo();
            return (0, path_1.join)(process.cwd(), gluestack_v2_1.GLUE_GENERATED_SERVICES_PATH, gatewayInstanceName, 'src', gatewayInstanceName, this.getName());
        }
    }
    exports.PluginInstance = PluginInstance;
});
